@@ -11,17 +11,18 @@
 #include "davmanager_p.h"
 #include "davprotocolbase_p.h"
 #include "davurl.h"
-#include "utils_p.h"
 #include "etagcache.h"
+#include "utils_p.h"
 
 #include <KIO/DavJob>
 #include <KIO/Job>
 
-#include <QSet>
+#include <set>
 
 using namespace KDAV;
 
-namespace KDAV {
+namespace KDAV
+{
 class DavItemsListJobPrivate : public DavJobBasePrivate
 {
 public:
@@ -33,7 +34,7 @@ public:
     QString mRangeStart;
     QString mRangeEnd;
     DavItem::List mItems;
-    QSet<QString> mSeenUrls; // to prevent events duplication with some servers
+    std::set<QString> mSeenUrls; // to prevent events duplication with some servers
     DavItem::List mChangedItems;
     QStringList mDeletedItems;
     uint mSubJobCount = 0;
@@ -68,10 +69,9 @@ void DavItemsListJob::start()
     Q_D(DavItemsListJob);
     const DavProtocolBase *protocol = DavManager::davProtocol(d->mUrl.protocol());
     Q_ASSERT(protocol);
-    QVectorIterator<XMLQueryBuilder::Ptr> it(protocol->itemsQueries());
 
-    while (it.hasNext()) {
-        XMLQueryBuilder::Ptr builder = it.next();
+    const auto queries = protocol->itemsQueries();
+    for (XMLQueryBuilder::Ptr builder : queries) {
         if (!d->mRangeStart.isEmpty()) {
             builder->setParameter(QStringLiteral("start"), d->mRangeStart);
         }
@@ -85,17 +85,21 @@ void DavItemsListJob::start()
         if (d->mMimeTypes.isEmpty() || d->mMimeTypes.contains(mimeType)) {
             ++d->mSubJobCount;
             if (protocol->useReport()) {
-                KIO::DavJob *job = DavManager::self()->createReportJob(d->mUrl.url(), props);
+                KIO::DavJob *job = DavManager::self()->createReportJob(d->mUrl.url(), props.toString());
                 job->addMetaData(QStringLiteral("PropagateHttpHeader"), QStringLiteral("true"));
                 job->setProperty("davType", QStringLiteral("report"));
                 job->setProperty("itemsMimeType", mimeType);
-                connect(job, &KIO::DavJob::result, this, [d](KJob *job) { d->davJobFinished(job); });
+                connect(job, &KIO::DavJob::result, this, [d](KJob *job) {
+                    d->davJobFinished(job);
+                });
             } else {
-                KIO::DavJob *job = DavManager::self()->createPropFindJob(d->mUrl.url(), props);
+                KIO::DavJob *job = DavManager::self()->createPropFindJob(d->mUrl.url(), props.toString());
                 job->addMetaData(QStringLiteral("PropagateHttpHeader"), QStringLiteral("true"));
                 job->setProperty("davType", QStringLiteral("propFind"));
                 job->setProperty("itemsMimeType", mimeType);
-                connect(job, &KIO::DavJob::result, this, [d](KJob *job) { d->davJobFinished(job); });
+                connect(job, &KIO::DavJob::result, this, [d](KJob *job) {
+                    d->davJobFinished(job);
+                });
             }
         }
     }
@@ -128,9 +132,9 @@ QStringList DavItemsListJob::deletedItems() const
 void DavItemsListJobPrivate::davJobFinished(KJob *job)
 {
     KIO::DavJob *davJob = qobject_cast<KIO::DavJob *>(job);
-    const int responseCode = davJob->queryMetaData(QStringLiteral("responsecode")).isEmpty()
-                             ? 0
-                             : davJob->queryMetaData(QStringLiteral("responsecode")).toInt();
+    const int responseCode = davJob->queryMetaData(QStringLiteral("responsecode")).isEmpty() //
+        ? 0
+        : davJob->queryMetaData(QStringLiteral("responsecode")).toInt();
 
     // KIO::DavJob does not set error() even if the HTTP status code is a 4xx or a 5xx
     if (davJob->error() || (responseCode >= 400 && responseCode < 600)) {
@@ -166,7 +170,8 @@ void DavItemsListJobPrivate::davJobFinished(KJob *job)
          */
 
         const QString itemsMimeType = job->property("itemsMimeType").toString();
-        const QDomDocument document = davJob->response();
+        QDomDocument document;
+        document.setContent(davJob->responseData(), true);
         const QDomElement documentElement = document.documentElement();
 
         QDomElement responseElement = Utils::firstChildElementNS(documentElement, QStringLiteral("DAV:"), QStringLiteral("response"));
@@ -194,7 +199,7 @@ void DavItemsListJobPrivate::davJobFinished(KJob *job)
 
             // check whether it is a DAV collection ...
             const QDomElement resourcetypeElement = Utils::firstChildElementNS(propElement, QStringLiteral("DAV:"), QStringLiteral("resourcetype"));
-            if (!responseElement.isNull()) {
+            if (!resourcetypeElement.isNull()) {
                 const QDomElement collectionElement = Utils::firstChildElementNS(resourcetypeElement, QStringLiteral("DAV:"), QStringLiteral("collection"));
                 if (!collectionElement.isNull()) {
                     responseElement = Utils::nextSiblingElementNS(responseElement, QStringLiteral("DAV:"), QStringLiteral("response"));
@@ -220,13 +225,13 @@ void DavItemsListJobPrivate::davJobFinished(KJob *job)
                 url = QUrl::fromUserInput(href);
             }
 
-            QString itemUrl = url.toDisplayString();
-            if (mSeenUrls.contains(itemUrl)) {
+            const QString itemUrl = url.toDisplayString();
+            const auto [it, isInserted] = mSeenUrls.insert(itemUrl);
+            if (!isInserted) {
                 responseElement = Utils::nextSiblingElementNS(responseElement, QStringLiteral("DAV:"), QStringLiteral("response"));
                 continue;
             }
 
-            mSeenUrls << itemUrl;
             auto _url = url;
             _url.setUserInfo(mUrl.url().userInfo());
             item.setUrl(DavUrl(_url, mUrl.protocol()));
@@ -246,10 +251,15 @@ void DavItemsListJobPrivate::davJobFinished(KJob *job)
         }
     }
 
-    const auto etagCacheUrls = mEtagCache->urls();
-    QSet<QString> removed(etagCacheUrls.begin(), etagCacheUrls.end());
-    removed.subtract(mSeenUrls);
-    mDeletedItems = removed.values();
+    mDeletedItems.clear();
+
+    const auto map = mEtagCache->etagCacheMap();
+    for (auto it = map.cbegin(); it != map.cend(); ++it) {
+        const QString remoteId = it.key();
+        if (mSeenUrls.find(remoteId) == mSeenUrls.cend()) {
+            mDeletedItems.append(remoteId);
+        }
+    }
 
     if (--mSubJobCount == 0) {
         emitResult();
